#Region " Imports"
Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Text
Imports System.Web
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.HtmlControls
Imports System.Diagnostics
Imports System.ComponentModel.Design
Imports System.ComponentModel.Design.Serialization
Imports System.Drawing
Imports System.Drawing.Design.UITypeEditor
Imports System.Drawing.Design
Imports System.Data.SqlClient
Imports System.IO
Imports System.Security.Permissions
Imports System.Xml
#End Region

<Assembly: WebResource("SuperGRID.Grade.js", "application/x-javascript", PerformSubstitution:=True)> 

Namespace SuperGridControl

    <AspNetHostingPermission(SecurityAction.InheritanceDemand, _
        Level:=AspNetHostingPermissionLevel.Minimal), AspNetHostingPermission(SecurityAction.Demand, Level:=AspNetHostingPermissionLevel.Minimal), ToolboxData("<{0}:SuperGrid runat=server></{0}:mySuperGRID>"), Description("Manipulate and display Grids of data/Controle para manipulao e exibio de grids.")> Public Class SuperGrid
        Inherits DataBoundControl
        Implements INamingContainer

#Region " Variables Declaration"

        Private _text As String
        Private _height As System.Web.UI.WebControls.Unit
        Private _columns As Columns
        Private _visible As Boolean = True
        Private _fixedLines As Integer = 3
        Private _dataSource As Object = Nothing
        Private _datamember As String = Nothing
        Private _automaticBackground As Boolean = True
        Private _width As System.Web.UI.WebControls.Unit
        Private _urlDeleteButton As String
        Private _urlAddButton As String

        Private _headerStyle As TableItemStyle
        Private _itemStyle As TableItemStyle
        Private _alternatingItemStyle As TableItemStyle
        Private _itemTemplate As ITemplate

        Private _showFooter As Boolean = True
        Private _footerAlignment As System.Web.UI.WebControls.HorizontalAlign = HorizontalAlign.Right

        Private _addLineTooltip As String = "Click here to add a new line"
        Private _deleteLineTooltip As String = "Click here to delete this line"

        Private _addLineLinkText As String = "Add"
        Private _deleteLineLinkText As String = "Remove"

#End Region

#Region " Properties"

        <Description("Add new line link text"), Browsable(True), NotifyParentProperty(True), Category("Appearance"), PersistenceMode(PersistenceMode.InnerProperty)> Property [AddLineTooLinkText]() As String
            Get
                Return _addLineLinkText
            End Get

            Set(ByVal Value As String)
                _addLineLinkText = Value
            End Set
        End Property

        <Description("Remove line link text"), Browsable(True), NotifyParentProperty(True), Category("Appearance"), PersistenceMode(PersistenceMode.InnerProperty)> Property [RemoveLineLinkText]() As String
            Get
                Return _deleteLineLinkText
            End Get

            Set(ByVal Value As String)
                _deleteLineLinkText = Value
            End Set
        End Property

        <Bindable(True), Category("Appearance"), NotifyParentProperty(True), DefaultValue(HorizontalAlign.Right)> Property [FooterAlignment]() As System.Web.UI.WebControls.HorizontalAlign
            Get
                Return _footerAlignment
            End Get

            Set(ByVal Value As System.Web.UI.WebControls.HorizontalAlign)
                _footerAlignment = Value
            End Set
        End Property

        <Description("Add new line tooltip"), Browsable(True), NotifyParentProperty(True), Category("Appearance"), PersistenceMode(PersistenceMode.InnerProperty)> Property [AddLineTooltip]() As String
            Get
                Return _addLineTooltip
            End Get

            Set(ByVal Value As String)
                _addLineTooltip = Value
            End Set
        End Property

        <Description("Remove line tooltip"), Browsable(True), NotifyParentProperty(True), Category("Appearance"), PersistenceMode(PersistenceMode.InnerProperty)> Property [RemoveLineTooltip]() As String
            Get
                Return _deleteLineTooltip
            End Get

            Set(ByVal Value As String)
                _deleteLineTooltip = Value
            End Set
        End Property

        <Description("Image of the delete button"), Browsable(True), NotifyParentProperty(True), RefreshProperties(RefreshProperties.Repaint), Editor(GetType(System.Web.UI.Design.ImageUrlEditor), GetType(System.Drawing.Design.UITypeEditor)), Category("Appearance"), PersistenceMode(PersistenceMode.InnerProperty)> Property [DeleteLineImage]() As String
            Get
                Return _urlDeleteButton
            End Get

            Set(ByVal Value As String)
                _urlDeleteButton = Replace(Value, "~/", "")
            End Set
        End Property

        <Description("Image of the add button"), Browsable(True), Editor(GetType(System.Web.UI.Design.ImageUrlEditor), GetType(System.Drawing.Design.UITypeEditor)), Category("Appearance"), PersistenceMode(PersistenceMode.InnerProperty)> Property [AddLineImage]() As String
            Get
                Return _urlAddButton
            End Get

            Set(ByVal Value As String)
                _urlAddButton = Replace(Value, "~/", "")
            End Set
        End Property

        <Description("Number of lines displayed when there is no datasource binded"), Category("Appearance"), DefaultValue(3), PersistenceMode(PersistenceMode.InnerProperty)> Property [FixedLines]() As Integer
            Get
                Return _fixedLines
            End Get

            Set(ByVal Value As Integer)
                _fixedLines = Value
            End Set
        End Property

        <Description("Show/Hide footer"), Category("Appearance"), DefaultValue(True), PersistenceMode(PersistenceMode.InnerProperty)> Property [ShowFooter]() As Boolean
            Get
                Return _showFooter
            End Get

            Set(ByVal Value As Boolean)
                _showFooter = Value
            End Set
        End Property

        <Description("Column definition"), Category("Appearance"), DesignerSerializationVisibility(DesignerSerializationVisibility.Content), PersistenceMode(PersistenceMode.InnerProperty)> ReadOnly Property [Columns]() As Columns
            Get

                If IsNothing(_columns) Then
                    _columns = New Columns
                End If
                Return _columns

            End Get
        End Property

        <Description("Control width"), Category("Layout"), DefaultValue(True)> Overrides Property [Width]() As System.Web.UI.WebControls.Unit

            Get
                Return _width
            End Get

            Set(ByVal Value As System.Web.UI.WebControls.Unit)
                _width = Value
            End Set

        End Property

        <Description("Show/hide the control"), Category("Appearance"), DefaultValue(True)> Overrides Property [Visible]() As Boolean
            Get
                Return _visible
            End Get

            Set(ByVal Value As Boolean)
                _visible = Value
            End Set
        End Property

        <Description("Sets the height of the control"), Category("Appearance"), DefaultValue("")> Overrides Property [Height]() As System.Web.UI.WebControls.Unit
            Get
                Return _height
            End Get

            Set(ByVal Value As System.Web.UI.WebControls.Unit)
                _height = Value
            End Set
        End Property

        <Bindable(True), _
           Category("Data"), _
           Description("Controls datasource."), _
           DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
        Public Overrides Property DataSource() As Object
            Get
                Return _dataSource
            End Get
            Set(ByVal Value As Object)
                _dataSource = Value
            End Set
        End Property

        <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), Description("Control datasource member"), Category("Data"), DefaultValue("")> Overrides Property [DataMember]() As String
            Get
                Return _datamember
            End Get

            Set(ByVal Value As String)
                _datamember = Value
            End Set
        End Property

        <Description("Sets if the control will render gray on the background of odd lines and white on the even"), Category("Appearance"), DefaultValue(True)> Property [AutomaticBackground]() As Boolean
            Get
                Return _automaticBackground
            End Get

            Set(ByVal Value As Boolean)
                _automaticBackground = Value
            End Set
        End Property

        <Category("Style"), _
           Description("Style to be applied in Alternating Items."), _
           DesignerSerializationVisibility(DesignerSerializationVisibility.Content), _
           NotifyParentProperty(True), _
           PersistenceMode(PersistenceMode.InnerProperty)> _
        Public Overridable ReadOnly Property AlternatingItemStyle() As TableItemStyle
            Get
                If _alternatingItemStyle Is Nothing Then
                    _alternatingItemStyle = New TableItemStyle
                    If IsTrackingViewState Then
                        CType(_alternatingItemStyle, IStateManager).TrackViewState()
                    End If
                End If
                Return _alternatingItemStyle
            End Get
        End Property

        <Bindable(True), _
           Category("Appearance"), _
           DefaultValue(-1), _
           Description("Tables cell pading.")> _
        Public Overridable Property CellPadding() As Integer
            Get
                If ControlStyleCreated = False Then
                    Return -1
                End If
                Return CType(ControlStyle, TableStyle).CellPadding
            End Get
            Set(ByVal Value As Integer)
                CType(ControlStyle, TableStyle).CellPadding = Value
            End Set
        End Property

        <Bindable(True), _
           Category("Appearance"), _
           DefaultValue(0), _
           Description("Tables cellspacing.")> _
        Public Overridable Property CellSpacing() As Integer
            Get
                If ControlStyleCreated = False Then
                    Return 0
                End If
                Return CType(ControlStyle, TableStyle).CellSpacing
            End Get
            Set(ByVal Value As Integer)
                CType(ControlStyle, TableStyle).CellSpacing = Value
            End Set
        End Property

        <Bindable(True), _
           Category("Appearance"), _
           DefaultValue(System.Web.UI.WebControls.GridLines.None), _
           Description("Gridlines of the control.")> _
        Public Overridable Property GridLines() As GridLines
            Get
                If ControlStyleCreated = False Then
                    Return GridLines.None
                End If
                Return CType(ControlStyle, TableStyle).GridLines
            End Get
            Set(ByVal Value As GridLines)
                CType(ControlStyle, TableStyle).GridLines = Value
            End Set
        End Property

        <Category("Style"), _
           Description("Header Style."), _
           DesignerSerializationVisibility(DesignerSerializationVisibility.Content), _
           NotifyParentProperty(True), _
           PersistenceMode(PersistenceMode.InnerProperty)> _
        Public Overridable ReadOnly Property HeaderStyle() As TableItemStyle
            Get
                If _headerStyle Is Nothing Then
                    _headerStyle = New TableItemStyle
                    If IsTrackingViewState Then
                        CType(_headerStyle, IStateManager).TrackViewState()
                    End If
                End If
                Return _headerStyle
            End Get
        End Property

        <Category("Style"), _
           Description("Style to be applied on all items."), _
           DesignerSerializationVisibility(DesignerSerializationVisibility.Content), _
           NotifyParentProperty(True), _
           PersistenceMode(PersistenceMode.InnerProperty)> _
        Public Overridable ReadOnly Property ItemStyle() As TableItemStyle
            Get
                If _itemStyle Is Nothing Then
                    _itemStyle = New TableItemStyle
                    If IsTrackingViewState Then
                        CType(_itemStyle, IStateManager).TrackViewState()
                    End If
                End If
                Return _itemStyle
            End Get
        End Property

#End Region

#Region " Methods"

#Region " Databind methods"

        Protected Overrides Sub PerformDataBinding(ByVal dataSource As IEnumerable)

            MyBase.PerformDataBinding(dataSource)

            If Not dataSource Is Nothing Then
                Call _BuildControl(dataSource)
            End If

        End Sub

#End Region

#Region " Control events"
        Protected Overrides Sub OnLoad(ByVal e As System.EventArgs)
            If Me._dataSource IsNot Nothing Then
                MyBase.OnLoad(e)
            End If
        End Sub
#End Region

#Region " Returning values methods"

        Public Function ReturnValues() As String
            Dim myReturnValues As New XmlDocument
            Dim xmlDeclar As XmlDeclaration = myReturnValues.CreateXmlDeclaration("1.0", "utf-8", Nothing)
            myReturnValues.InsertBefore(xmlDeclar, myReturnValues.DocumentElement)

            Dim rootNode As XmlElement = myReturnValues.CreateElement("resultList")
            myReturnValues.AppendChild(rootNode)

            Dim x As String = HttpContext.Current.Request.Form(Me.ClientID)

            Dim parentNode As XmlElement = myReturnValues.CreateElement("Line")
            rootNode.AppendChild(parentNode)

            '// Create the required nodes
            Dim nameNode As XmlElement = myReturnValues.CreateElement("controlName")
            Dim valueNode As XmlElement = myReturnValues.CreateElement("controlValue")

            Dim controlName As XmlText = myReturnValues.CreateTextNode("control name")
            Dim controlValue As XmlText = myReturnValues.CreateTextNode("control value")

            parentNode.AppendChild(nameNode)
            parentNode.AppendChild(valueNode)

            nameNode.AppendChild(controlName)
            valueNode.AppendChild(controlValue)

            Return myReturnValues.InnerXml
        End Function


#End Region

#End Region

#Region " Renderization"

        Private Sub _BuildControl(Optional ByVal retrievedData As IEnumerable = Nothing)
            Controls.Clear()

            Dim LineIndex As Integer = 1
            Dim myTable As New Table

            With myTable
                .Attributes.Item("rules") = "rows"
                .GridLines = CType(ControlStyle, TableStyle).GridLines
                If Not _width.IsEmpty Then
                    .Width = _width.ToString
                End If
                .CellPadding = CellPadding
                .CellSpacing = CellSpacing
                .ID = "Grid"
                .Attributes.Item("name") = Me.ClientID & "_Grid"
            End With

            If Not _columns Is Nothing Then
                'method to create the header
                myTable.Rows.Add(CreateHeader())
            End If

            Dim Buiding As Boolean = True

            While Buiding
                If retrievedData Is Nothing Then 'nao tem datasource, montar em branco
                    For LineIndex = 1 To _fixedLines + 1
                        myTable.Rows.Add(CreateDetailLine(LineIndex))
                    Next
                    Buiding = False
                Else
                    Dim dataLine As Object
                    myTable.Rows.Add(CreateDetailLine(LineIndex))
                    LineIndex = 2
                    For Each dataLine In retrievedData
                        myTable.Rows.Add(CreateDetailLine(LineIndex, dataLine))
                        LineIndex += 1
                    Next
                    Buiding = False
                End If
            End While

            If _showFooter Then
                If Not _columns Is Nothing Then
                    myTable.Rows.Add(CreateFooter(_columns.Count, Me.ClientID & "_" & myTable.ClientID))
                Else
                    myTable.Rows.Add(CreateFooter(1, Me.ClientID & "_" & myTable.ClientID))
                End If
            End If

            Controls.Add(myTable)
        End Sub

        Private Sub ApplyAutomaticBackground(ByRef cell As TableCell, ByVal LineIndex As Integer)
            If _automaticBackground Then
                If LineIndex Mod 2 = 1 Then
                    If Not _itemStyle Is Nothing Then
                        cell.ApplyStyle(_itemStyle)
                    End If
                Else
                    If Not _alternatingItemStyle Is Nothing Then
                        cell.ApplyStyle(_alternatingItemStyle)
                    End If
                End If
            End If
        End Sub

        Protected Overrides Sub CreateChildControls()

            If Me._dataSource Is Nothing Then
                Call _BuildControl()
            End If
            Return

        End Sub

        Private Function CreateHeader() As TableRow

            Dim ColumnIndex As Integer

            Dim myTable As New Table
            Dim myLine As New TableRow

            For ColumnIndex = 0 To _columns.Count - 1
                Dim myCell As New TableCell
                If _columns(ColumnIndex).Visible Then
                    myCell.Text = _columns(ColumnIndex).HeaderText
                End If
                If ColumnIndex = _columns.Count - 1 Then
                    myCell.ColumnSpan = 2
                End If
                If Not _columns(ColumnIndex).Width.IsEmpty Then
                    myCell.Width = _columns(ColumnIndex).Width.ToString
                End If

                If Not _headerStyle Is Nothing Then
                    Dim x As String = _headerStyle.ToString
                    myCell.ApplyStyle(_headerStyle)
                Else
                    myCell.BackColor = System.Drawing.Color.Navy
                    myCell.Font.Bold = True
                    myCell.ForeColor = System.Drawing.Color.White
                    myTable.CellPadding = -1
                    myTable.CellSpacing = 0
                End If
                myLine.Cells.Add(myCell)
            Next

            Return myLine
        End Function

        Private Function CreateDeleteCell() As TableCell
            Dim removeCell As New TableCell
            Dim removeButton As New HtmlImage
            removeButton.Alt = _deleteLineTooltip
            If Not _urlAddButton Is Nothing Then
                removeButton.Src = Page.ResolveUrl(_urlDeleteButton.ToString)
            End If
            removeButton.Style.Item("Cursor") = "HAND"
            removeButton.Attributes.Item("title") = _deleteLineTooltip
            removeButton.Attributes.Item("onclick") = "deleteLine(this);return false;"

            Dim link As New LinkButton
            'link.Attributes.Item("onclick") = "ExcluiLinha(this);return false;"
            link.OnClientClick = "deleteLine(this);return false;"
            link.Text = _deleteLineLinkText

            'TODO Here we have to create a link control if we don't have an image
            If Not _urlAddButton Is Nothing Then
                removeCell.Controls.Add(removeButton)
            Else
                removeCell.Controls.Add(link)
            End If
            Return removeCell

        End Function

        Private Function CreateFooter(ByVal ColumnCount As Long, ByVal clientID As String) As TableRow
            Dim newLine As New TableRow
            Dim myCell As New TableCell
            myCell.Attributes.Item("ColumnSpan") = ColumnCount + 1
            Dim addButton As New HtmlImage
            If Not _urlAddButton Is Nothing Then
                addButton.Src = _urlAddButton.ToString
            End If

            addButton.ID = Me.ClientID & "_btnAddNewLine"
            addButton.Style.Item("Cursor") = "HAND"
            addButton.Attributes.Item("title") = _addLineTooltip
            addButton.Attributes.Item("onclick") = "duplicateLine('" & clientID & "');return false;"

            Dim link As New LinkButton
            'link.Attributes.Item("onclick") = "javascript:duplicateLine('" & clientID & "');return false;"
            link.OnClientClick = "duplicateLine('" & clientID & "');return false;"
            link.Text = _addLineLinkText

            If Not _urlAddButton Is Nothing Then
                myCell.Controls.Add(addButton)
            Else
                myCell.Controls.Add(link)
            End If
            myCell.Attributes.Item("align") = _footerAlignment.ToString
            newLine.Controls.Add(myCell)
            Return newLine
        End Function

        Private Function CreateDetailLine(ByVal LineIndex As Integer, Optional ByVal dataLine As Object = Nothing) As TableRow
            Dim myLine As New TableRow
            Dim ColumnIndex As Integer

            If LineIndex = 1 Then
                myLine.Style.Item("display") = "none"
            End If

            If Not _columns Is Nothing Then
                For ColumnIndex = 0 To _columns.Count - 1
                    Dim objValue As Object = Nothing
                    If Me.DesignMode Then
                        If Not String.IsNullOrEmpty(_columns(ColumnIndex).DataTextField) Then
                            objValue = "Databound: [" + _columns(ColumnIndex).DataTextField + "]"
                        End If
                    Else
                        If Not dataLine Is Nothing AndAlso Not _columns(ColumnIndex).DataTextField Is Nothing AndAlso _columns(ColumnIndex).DataTextField <> "" Then
                            objValue = DataBinder.GetPropertyValue(dataLine, _columns(ColumnIndex).DataTextField)
                        End If
                    End If

                    Dim myCell As New TableCell
                    If Not _columns(ColumnIndex).Width.IsEmpty Then
                        myCell.Width = _columns(ColumnIndex).Width.ToString
                    End If
                    If Not _columns(ColumnIndex).Height.IsEmpty Then
                        myCell.Height = _columns(ColumnIndex).Height.ToString
                    End If

                    myCell.HorizontalAlign = _columns(ColumnIndex).HorizontalAlignment
                    myCell.VerticalAlign = _columns(ColumnIndex).VerticalAlignment
                    Call ApplyAutomaticBackground(myCell, LineIndex)

                    Dim ControlIndex As Long
                    For ControlIndex = 0 To _columns(ColumnIndex).Controls.Count - 1
                        myCell.Controls.Add(CreateControls(_columns(ColumnIndex).Controls(ControlIndex), objValue, LineIndex, ColumnIndex))
                    Next
                    If Not _columns(ColumnIndex).Visible Then
                        myCell.Style.Item("display") = "none"
                    End If
                    myLine.Cells.Add(myCell)
                    If ColumnIndex = _columns.Count - 1 Then
                        myLine.Cells.Add(CreateDeleteCell)
                    End If
                Next
            End If

            Return myLine
        End Function

        Private Function CreateControls(ByVal myControl As ColumnControl, ByVal objValue As Object, ByVal lineIndex As Long, ByVal columnIndex As Long) As Object

            Dim myField As Object = Nothing

            'settings by type
            Select Case myControl.ControlType
                Case Is = ColumnControl.TypeOfControl.Label
                    If Not objValue Is Nothing Then
                        myField = objValue
                    End If
                Case Is = ColumnControl.TypeOfControl.CheckBox
                    myField = New HtmlInputCheckBox
                    myField.Attributes.Item("width") = "100%"
                    myField.ID = myControl.ID & lineIndex & "_" & (columnIndex + 1).ToString
                Case Is = ColumnControl.TypeOfControl.Link

                Case Is = ColumnControl.TypeOfControl.OptionBox
                    myField = New HtmlInputRadioButton
                    myField.Attributes.Item("width") = "100%"
                    myField.ID = myControl.ID & lineIndex & "_" & (columnIndex + 1).ToString
                    If Not objValue Is Nothing Then
                        myField.Checked = objValue
                    End If
                Case Is = ColumnControl.TypeOfControl.Text
                    myField = New HtmlControls.HtmlInputText
                    myField.id = myControl.ID & lineIndex & "_" & (columnIndex + 1).ToString
                    With myField
                        If Not myControl.Width.IsEmpty Then
                            myField.Attributes.Item("width") = myControl.Width.ToString
                        End If
                        If Not objValue Is Nothing Then
                            .Text = objValue
                            .Attributes.Item("originalValue") = objValue
                        Else
                            .Attributes.Item("originalValue") = ""
                        End If
                    End With
                Case Is = ColumnControl.TypeOfControl.DropdownList
                    myField = New DropDownList
                    myField.Attributes.Item("ID") = myControl.ID
                    myField.Attributes.Item("width") = myControl.Width
                    If Not myControl.DropdownDataSource Is Nothing Then
                        myField.DataSource = myControl.DropdownDataSource
                    End If
                    If Not objValue Is Nothing Then
                        myField.Items.FindByText(objValue)
                    End If
            End Select

            'general settings
            If myControl.Visible AndAlso myControl.Enabled Then
                If myControl.OnChangeEvent IsNot Nothing AndAlso myControl.OnChangeEvent.Trim <> "" Then
                    myField.Attributes.Item("onchange") = myControl.OnChangeEvent
                End If
                If myControl.OnClickEvent IsNot Nothing AndAlso myControl.OnClickEvent.Trim <> "" Then
                    myField.Attributes.Item("onclick") = myControl.OnClickEvent
                End If
                If myControl.OnBlurEvent IsNot Nothing AndAlso myControl.OnBlurEvent.Trim <> "" Then
                    myField.Attributes.Item("onblur") = myControl.OnBlurEvent
                End If
                If myControl.OnKeyDownEvent IsNot Nothing AndAlso myControl.OnKeyDownEvent.Trim <> "" Then
                    myField.Attributes.Item("onkeydown") = myControl.OnKeyDownEvent
                End If
                If myControl.OnKeyPressEvent IsNot Nothing AndAlso myControl.OnKeyPressEvent.Trim <> "" Then
                    myField.Attributes.Item("onkeypress") = myControl.OnKeyPressEvent
                End If
            ElseIf myControl.Visible AndAlso Not myControl.Enabled Then
                myField.Attributes.Item("disabled") = "true"
            End If

            myField.Attributes.Item("internalName") = myControl.ID

            'if first line then this will be the fixed hiddended line to be used on duplicate line
            If lineIndex = 1 Then
                myField.Attributes.Item("masterLine") = "true"
            Else
                myField.Attributes.Item("masterLine") = "false"
            End If

            Return myField
        End Function

        Protected Overrides Function CreateControlStyle() As Style
            Dim style As New TableStyle(ViewState)
            style.CellSpacing = 0
            Return style
        End Function

        Protected Overrides Sub OnPreRender(ByVal e As System.EventArgs)

            'Javascript webresource registration
            Dim RecurseName As String = "SuperGRID.Grade.js"
            Dim RecurseType As Type = GetType(SuperGridControl.SuperGrid)
            Page.ClientScript.RegisterClientScriptResource(RecurseType, RecurseName)
            Page.ClientScript.RegisterOnSubmitStatement(Me.GetType, "generateXML", "buildReturn('" & Me.ClientID & "');")

        End Sub

        Protected Overrides Sub Render(ByVal writer As System.Web.UI.HtmlTextWriter)

            Call RenderContents(writer)

        End Sub

#End Region

    End Class

End Namespace

